﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ReprojectionProblemTest
{
    public partial class ReprojectionProblemForm : Form
    {
        public ReprojectionProblemForm()
        {
            InitializeComponent();


            SetupForProjectionTest();

        }

        public string ShapefileToReproject { get { return Path.Combine(Application.StartupPath, "1d_nwk_Dummy_Culv_L.shp"); } }

        public string BaseShapefile { get { return Path.Combine(Application.StartupPath, "River_Active_Area.shp"); } }

        

        public void SetupForProjectionTest()
        {
            // Set reprojection settings
            var setting = new MapWinGIS.GlobalSettings();
            setting.AllowLayersWithoutProjections = true;
            setting.AllowLayersWithIncompleteReprojection = true; 
            setting.ReprojectLayersOnAdding = true;
            setting.AllowProjectionMismatch = true;

            // Set Projection events
            axMap1.ProjectionChanged += AxMap1_ProjectionChanged;
            axMap1.ProjectionMismatch += AxMap1_ProjectionMismatch;
            axMap1.LayerReprojectedIncomplete += AxMap1_LayerReprojectedIncomplete;
            axMap1.LayerReprojected += AxMap1_LayerReprojected;
            axMap1.LayerProjectionIsEmpty += AxMap1_LayerProjectionIsEmpty;


            var mapGP = new MapWinGIS.GeoProjection();
            var shpGP = new MapWinGIS.GeoProjection();

            // Set the map projection from our "base" shapefile - EPSG 27700 - British National Grid - this works
            bool success = mapGP.ImportFromWKT(File.ReadAllText(Path.ChangeExtension(BaseShapefile, ".prj")));
            //this.axMap1.GeoProjection = mapGP; - setting map projection like this offsets the shapefile, which is drawn slightly out of place
            axMap1.GrabProjectionFromData = true; // - setting the map projection like this positions the shapefile correctly.

            // Check the projection we want to convert FROM, from the .prj file - this file is WGS 84 UTM_60S - EPSG Code 32760 - this works
            success = shpGP.ImportFromWKT(File.ReadAllText(Path.ChangeExtension(ShapefileToReproject, ".prj")));

            
            // Add the base shapefile in EPSG 27700 - British National Grid format - this should work - single polygon
            int handle = axMap1.AddLayerFromFilename(BaseShapefile, MapWinGIS.tkFileOpenStrategy.fosAutoDetect, true);


            // add the file we want to project - this will fail - a single line
            handle = axMap1.AddLayerFromFilename(ShapefileToReproject, MapWinGIS.tkFileOpenStrategy.fosAutoDetect, true);


            axMap1.ZoomToMaxExtents();

        }

        private void AxMap1_LayerReprojectedIncomplete(object sender, AxMapWinGIS._DMapEvents_LayerReprojectedIncompleteEvent e)
        {
            Debug.WriteLine($"Incomplete, num projected = {e.numReprojected}");
        }

        private void AxMap1_LayerProjectionIsEmpty(object sender, AxMapWinGIS._DMapEvents_LayerProjectionIsEmptyEvent e)
        {
            Debug.WriteLine($"Projection is empty, cancel adding = {e.cancelAdding}, layer handle = {e.layerHandle}");
        }

        private void AxMap1_LayerReprojected(object sender, AxMapWinGIS._DMapEvents_LayerReprojectedEvent e)
        {
            Debug.WriteLine($"Layer reprojection successful {e.success}");
        }


        private void AxMap1_ProjectionMismatch(object sender, AxMapWinGIS._DMapEvents_ProjectionMismatchEvent e)
        {
            Debug.WriteLine($"Projection mismatch found, reproject = {e.reproject}, cancel adding = {e.cancelAdding}");
        }

        private void AxMap1_ProjectionChanged(object sender, EventArgs e)
        {
            Debug.WriteLine("Layer projection changed");
        }
    }
}
